package com.uoc.pfc.catalogoweb.service.ibatis;

import java.io.Reader;
import java.util.HashMap;
import java.util.Map;

import com.ibatis.common.resources.Resources;
import com.ibatis.sqlmap.client.SqlMapClient;
import com.ibatis.sqlmap.client.SqlMapClientBuilder;

public class IbatisLocator {
    
	/**
	 * Nombre de la clave para almacenar el nombre del fichero de configuracin de Ibatis
	 * en una propiedad del sistema
	 */
	private static final String IBATIS_CONFIG_FILE_PROPERTY = "com.uoc.pfc.catalogoweb.ibatis.config";
	
	/** Nombre del fichero de configuracin de Ibatis por defecto */
	private static final String DEFAULT_IBATIS_CONFIG_FILE = "sql-map-config.xml";
	
	/** Contenedor de los distintos SqlMapClient */
	private Map sqlMap;
	
	/** Atributo esttico de tipo IbatisLocator para implementar el patrn singleton. */
	private static IbatisLocator ibatisLocator;
	
	/**
	 * Constructor que se ejecutar una sola vez para crear la 
	 * instancia de SqlMapClient que se usar en la aplicacin.
	 */
	private IbatisLocator() {

		String configFile = System.getProperty(IBATIS_CONFIG_FILE_PROPERTY, DEFAULT_IBATIS_CONFIG_FILE);
		
		try {
			sqlMap = new HashMap();
			sqlMap.put(DEFAULT_IBATIS_CONFIG_FILE, 
					   SqlMapClientBuilder.buildSqlMapClient(
					   		getReader(configFile)));
			
		} catch (Exception e) {
			System.out.println("Error creating SqlMapClient from file " + configFile);
		}				
	}
	
	/**
	 * Devuelve un objeto de tipo IbatisLocator;
	 * @return IbatisLocator
	 */
	public static IbatisLocator getInstance() {
		
		if (ibatisLocator == null) {
			ibatisLocator = new IbatisLocator();
		}
		return ibatisLocator;
	}
	
	/**
	 * Devuelve un SqlMapClient
	 * @return SqlMapClient
	 */
	public SqlMapClient getSqlMapClient() {
		
		if (sqlMap == null) {

			throw new NullPointerException("Ibatis Locator not initialized");
		}
		return (SqlMapClient) sqlMap.get(DEFAULT_IBATIS_CONFIG_FILE);
	}

	/**
	 * Devuelve un SqlMapClient
	 * @param aKey nombre de la clave asociada a un fichero de configuracin
	 * @return SqlMapClient asociado a dicha key
	 */
	public SqlMapClient getSqlMapClient(String aKey) {
		
		SqlMapClient sqlMapClient = null;
		
		if ((aKey != null) && (! aKey.equals(""))) {
			
			if (sqlMap != null) {
				
				if (! sqlMap.containsKey(aKey)) {

					sqlMap.put(aKey, SqlMapClientBuilder.buildSqlMapClient(getReader(aKey)));
				}
				
				sqlMapClient = (SqlMapClient) sqlMap.get(aKey);
				
			} else {

				throw new NullPointerException("Ibatis Locator not initialized");
			}
			
		} else {

			throw new IllegalArgumentException("Configuration File not exist: " + aKey);
		}
		
		return sqlMapClient;
	}
		
	/**
	 * Obtencin del Reader asociado a un determinado fichero
	 * @param aConfigFile nombre del fichero
	 * @return Reader del fichero
	 */
	private Reader getReader(String aConfigFile) {

		Reader reader = null;
		
		try {
			reader = Resources.getResourceAsReader(aConfigFile);
			
		} catch (Exception e) {
			throw new RuntimeException("Configuration File not exist: " + aConfigFile);
		}				
		
		return reader;
	}
        
}
